from __future__ import with_statement
import spss, spssaux, spssdata
import pdb


def Run(args):
    helptext="""The CEM command requires the R Integration Plug-in, the Python
Integration Plug-in, and the R cem package.

CEM  TREATMENT=variable VARIABLES=variable list
[/NOCOARSENING VARIABLES=variable list]
[/CUTPOINTS cutlist] [/GROUPINGS grouplist]
[/OPTIONS [K2K] [NOIMBAL]]

Example:
  RCEM TREATMENT=y VARIABLES=x1 x2 x3

TREATMENT is the treatment variable of interest (that is, the variable whose
effect you would like to measure). VARIABLES are the variables on which to
match. This set should include any variable that affects both the treatment
and the outcome (but not the outcome itself!).

/NOCOARSENING VARIABLES causes CEM to leave the specified variables entact in
the matching. Thus, these variables are not coarsened and are matched exactly.

/CUTPOINTS allows users to set custom cutpoints for coarsening of individual
variables. The 'cutlist' should have the following syntax:

  /CUTPOINTS \"var1 = (cut1, cut2, cut3, cut4)\" \"var2 = (cut5, cut6, cut7)\"

Here, \"var1\" and \"var2\" are variables names to have custom cutpoints at the
values inside the parentheses. Note that the first and last cutpoints should be
below the minimum and above the maximum, respectively.

/GROUPINGS allows users to set custom groupings for coarsening of categorical
variables. The 'grouplist' should have the following syntax:

  /GROUPINGS \"var1 = [(value1, value2), (value3, value4)]\" \"var2 = [(value5), (value6, value7)]\"

Here, \"var1\" and \"var2\" are variables names to have custom groupings.
Brackets [] enclose the list of groupings for a variable and parentheses ()
denote individual groupings for the variable. Again, the groupings for each
variable should be enclosed in quotes.

/OPTIONS K2K tells CEM to balance the number of treatment and control units
matched. Thus, within matching strata, CEM will randomly prune cases until there
are equal numbers of treatment and control.

/OPTIONS NOIMBAL will force CEM to omit the calculation of the imbalance after
matching. This can speed up the algorithm with extremely large datasets.

CEM /HELP prints this information and does nothing else.
"""
    dataname = spss.ActiveDataset()
    args = args[args.keys()[0]]

    if (any(x == "HELP" for x in args.keys())):
        print helptext
        return helptext

    try:
        spss.Submit("BEGIN PROGRAM R.\nlibrary(cem)\nEND PROGRAM R.")
    except:
        print("\n*** The cem packages is not installed in R.\n")

    ## tidy up the call to the backend
    values = ["RCEM"]
    for i in args.keys(): ## loop over subcommands
        if i == "":
            values.append(i)
        else:
            values.append("\n/"+i)
        for j in args[i]: ## loop over parameters
            if (len(j[j.keys()[0]]) > 0): ## parameter has an equals sign
                if (j.keys()[0] == "TOKENLIST"):
                    values.append("\"" + "\" \"".join(j.values()[0]) + "\"")
                else:
                    values.append(j.keys()[0] + "=" + " ".join(j.values()[0]))
            else:
                values.append(j.keys()[0])
    cmd = " ".join(values) + "\n/SAVE OUTPUTDATA=cemoutdata."
    #print cmd
    spss.Submit(cmd)

    ## Switch to cem output to grab data
    with spss.DataStep():
        newds = spss.Dataset(name = "cemoutdata")
        spss.SetActive(newds)
    cur = spss.Cursor([0])
    cemMatched = cur.fetchall()
    cur.close()
    cur = spss.Cursor([1])
    cemStrata = cur.fetchall()
    cur.close()
    cur = spss.Cursor([2])
    cemWeights = cur.fetchall()
    cur.close()

    ## need list of lists to assign to data below
    cemMatched = [[x[0]] for x in cemMatched]
    cemStrata = [[x[0]] for x in cemStrata]
    cemWeights = [[x[0]] for x in cemWeights]
    with spss.DataStep():
        oldds = spss.Dataset(name = dataname)
        spss.SetActive(oldds)

        ## Delete old cem variables
        oldlist = oldds.varlist
        if (any(n.name == 'cem.matched' for n in oldlist)):
            del oldlist['cem.matched']
        if (any(n.name == 'cem.strata' for n in oldlist)):            
            del oldlist['cem.strata']
        if (any(n.name == 'cem.weights' for n in oldlist)):            
            del oldlist['cem.weights']

        ## Setup new cem variables
        oldcases = oldds.cases
        nobs = len(oldcases)
        oldlist.append('cem.matched', 0)
        oldlist['cem.matched'].measurementLevel = 'nominal'
        oldlist['cem.matched'].label = 'Unit was matched by CEM'
        oldlist['cem.matched'].format = (5,8,0)
        
        oldlist.append('cem.strata', 0)
        oldlist['cem.strata'].measurementLevel = 'nominal'
        oldlist['cem.strata'].label = 'CEM Strata'
        oldlist['cem.strata'].format = (5,8,0)
        
        oldlist.append('cem.weights', 0)
        oldlist['cem.weights'].measurementLevel = 'scale'
        oldlist['cem.weights'].label = 'CEM Weights'
        oldlist['cem.weights'].format = (5,8,4)

        oldcases[0:nobs, oldlist['cem.matched'].index] = cemMatched
        oldcases[0:nobs, oldlist['cem.strata'].index] = cemStrata
        oldcases[0:nobs, oldlist['cem.weights'].index] = cemWeights
    spss.Submit('DATASET CLOSE cemoutdata.')
